#!/usr/bin/env python

"""
Type deduction operations.

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import sys

try:
    set
except NameError:
    from sets import Set as set

def get_object_types_for_usage(usage, objtable, name, unit_name, all_attributes, importer=None):

    """
    Return for the given attribute 'usage', using the 'objtable', the object
    types which satisfy such usage, reporting any problems for the given 'name'
    and 'unit_name'. Each object type is given as a tuple of the form (type,
    is_static).

    Where 'all_attributes' is set to a true value, all attributes in a usage
    list must be matched to provide object types. Otherwise, the presence of any
    attribute from a usage list in a type's set of attributes will be enough to
    provide that type as a suitable object type.

    If 'importer' is specified and not None, it will be used to record warnings
    about unsatisfiable usage situations.
    """

    all_objtypes = set()

    for attrnames in usage:
        attrnames = attrnames or ()

        # Determine whether all attributes are required.

        if all_attributes:
            objtypes = objtable.all_possible_objects_plus_status(attrnames)
        else:
            objtypes = objtable.any_possible_objects_plus_status(attrnames)
        if not objtypes and importer:
            importer.add_usage_failure(all_attributes, unit_name, name, attrnames.keys())

        all_objtypes.update(objtypes)

    return all_objtypes

def filter_using_self(objtypes, cls):

    """
    Filter the given 'objtypes' (a collection of object names and static
    indicators) for the given 'cls', returning only the (name, static) elements
    compatible with 'cls' and its descendants.
    """

    descendants = cls.all_descendants()
    filtered = []
    for objname, is_static in objtypes:
        if objname == cls.full_name() or objname in descendants:
            filtered.append((objname, is_static))
    return filtered

# vim: tabstop=4 expandtab shiftwidth=4
